@@ -368,9 +368,17 @@ namespace xsimd
 
     inline void batch<double, 2>::store_aligned(uint16_t* dst) const
     {
+#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
         __m128i tmp = _mm_cvtpd_epi32(m_value);
         __m128i tmp1 = _mm_packus_epi32(tmp, tmp);
         _mm_storel_epi64((__m128i*)dst, tmp1);
+#else
+        alignas(16) double tmp[2];
+        _mm_store_pd(tmp, m_value);
+        unroller<2>([&](std::size_t i){
+            dst[i] = static_cast<uint16_t>(tmp[i]);
+        });
+#endif
     }
 
     inline void batch<double, 2>::store_unaligned(uint16_t* dst) const
