@@ -89,77 +89,33 @@ namespace xsimd
     };
 
     template <>
-    class batch<int64_t, 2> : public simd_batch<batch<int64_t, 2>>
+    class batch<int64_t, 2> : public sse_int_batch<int64_t, 2>
     {
     public:
 
-        using self_type = batch<int64_t, 2>;
-        using base_type = simd_batch<self_type>;
-
-        batch();
-        explicit batch(int64_t i);
-        batch(int64_t i0, int64_t i1);
-        explicit batch(const int64_t* src);
-        batch(const int64_t* src, aligned_mode);
-        batch(const int64_t* src, unaligned_mode);
-        batch(const __m128i& rhs);
-        batch& operator=(const __m128i& rhs);
-
-        operator __m128i() const;
-
-        batch& load_aligned(const int64_t* src);
-        batch& load_unaligned(const int64_t* src);
-
-        batch& load_aligned(const int32_t* src);
-        batch& load_unaligned(const int32_t* src);
-
-        batch& load_aligned(const float* src);
-        batch& load_unaligned(const float* src);
-
-        batch& load_aligned(const double* src);
-        batch& load_unaligned(const double* src);
-
-        batch& load_aligned(const int8_t* src);
-        batch& load_unaligned(const int8_t* src);
-
-        batch& load_aligned(const uint8_t* src);
-        batch& load_unaligned(const uint8_t* src);
-
-        void store_aligned(int64_t* dst) const;
-        void store_unaligned(int64_t* dst) const;
-
-        void store_aligned(int32_t* dst) const;
-        void store_unaligned(int32_t* dst) const;
-
-        void store_aligned(float* dst) const;
-        void store_unaligned(float* dst) const;
-
-        void store_aligned(double* dst) const;
-        void store_unaligned(double* dst) const;
-
-        void store_aligned(int8_t* dst) const;
-        void store_unaligned(int8_t* dst) const;
-
-        void store_aligned(uint8_t* dst) const;
-        void store_unaligned(uint8_t* dst) const;
-
+        using base_type = sse_int_batch<int64_t, 2>;
+        using base_type::base_type;
         using base_type::load_aligned;
         using base_type::load_unaligned;
         using base_type::store_aligned;
         using base_type::store_unaligned;
 
-        int64_t operator[](std::size_t index) const;
-
-    private:
-
-        __m128i m_value;
-    };
+        XSIMD_DECLARE_LOAD_STORE_INT64(int64_t, 2);
+   };
 
     template <>
     class batch<uint64_t, 2> : public sse_int_batch<uint64_t, 2>
     {
     public:
-        using sse_int_batch::sse_int_batch;
+
+        using base_type = sse_int_batch<uint64_t, 2>;
+        using base_type::base_type;
+        using base_type::load_aligned;
+        using base_type::load_unaligned;
+        using base_type::store_aligned;
+        using base_type::store_unaligned;
+
+        XSIMD_DECLARE_LOAD_STORE_INT64(uint64_t, 2);
     };
 
     batch<int64_t, 2> operator<<(const batch<int64_t, 2>& lhs, int32_t rhs);
@@ -171,226 +127,134 @@ namespace xsimd
      * batch<int64_t, 2> implementation *
      ************************************/
 
-    inline batch<int64_t, 2>::batch()
+    namespace sse_detail
     {
-    }
-
-    inline batch<int64_t, 2>::batch(int64_t i)
-        : m_value(_mm_set1_epi64x(i))
-    {
-    }
-
-    inline batch<int64_t, 2>::batch(int64_t i0, int64_t i1)
-        : m_value(_mm_set_epi64x(i1, i0))
-    {
-    }
-
-    inline batch<int64_t, 2>::batch(const int64_t* src)
-        : m_value(_mm_loadu_si128((__m128i const*)src))
-    {
-    }
-
-    inline batch<int64_t, 2>::batch(const int64_t* src, aligned_mode)
-        : m_value(_mm_load_si128((__m128i const*)src))
-    {
-    }
-
-    inline batch<int64_t, 2>::batch(const int64_t* src, unaligned_mode)
-        : m_value(_mm_loadu_si128((__m128i const*)src))
-    {
-    }
-
-    inline batch<int64_t, 2>::batch(const __m128i& rhs)
-        : m_value(rhs)
-    {
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::operator=(const __m128i& rhs)
-    {
-        m_value = rhs;
-        return *this;
-    }
-
-    inline batch<int64_t, 2>::operator __m128i() const
-    {
-        return m_value;
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_aligned(const int64_t* src)
-    {
-        m_value = _mm_load_si128((__m128i const*)src);
-        return *this;
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_unaligned(const int64_t* src)
-    {
-        m_value = _mm_loadu_si128((__m128i const*)src);
-        return *this;
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_aligned(const int32_t* src)
-    {
-        alignas(16) int64_t tmp[2];
-        tmp[0] = int64_t(src[0]);
-        tmp[1] = int64_t(src[1]);
-        m_value = load_aligned(tmp);
-        return *this;
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_unaligned(const int32_t* src)
-    {
-        return load_aligned(src);
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_aligned(const float* src)
-    {
-        alignas(16) int64_t tmp[2];
-        tmp[0] = static_cast<int64_t>(src[0]);
-        tmp[1] = static_cast<int64_t>(src[1]);
-        m_value = load_aligned(tmp);
-        return *this;
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_unaligned(const float* src)
-    {
-        return load_aligned(src);
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_aligned(const double* src)
-    {
-        alignas(16) int64_t tmp[2];
-        tmp[0] = static_cast<int64_t>(src[0]);
-        tmp[1] = static_cast<int64_t>(src[1]);
-        m_value = load_aligned(tmp);
-        return *this;
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_unaligned(const double* src)
-    {
-        return load_aligned(src);
-    }
-
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_aligned(const int8_t* src)
-    {
-        __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
+        inline __m128i load_aligned_int64(const int8_t* src)
+        {
+            __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
-        m_value = _mm_cvtepi8_epi64(tmp);
+            __m128i res = _mm_cvtepi8_epi64(tmp);
 #else
-        __m128i mask = _mm_cmplt_epi8(tmp, _mm_set1_epi8(0));
-        __m128i tmp1 = _mm_unpacklo_epi8(tmp, mask);
-        mask = _mm_cmplt_epi16(tmp1, _mm_set1_epi16(0));
-        __m128i tmp2 = _mm_unpacklo_epi16(tmp1, mask);
-        mask = _mm_cmplt_epi32(tmp2, _mm_set1_epi32(0));
-        m_value = _mm_unpacklo_epi32(tmp2, mask);
+            __m128i mask = _mm_cmplt_epi8(tmp, _mm_set1_epi8(0));
+            __m128i tmp1 = _mm_unpacklo_epi8(tmp, mask);
+            mask = _mm_cmplt_epi16(tmp1, _mm_set1_epi16(0));
+            __m128i tmp2 = _mm_unpacklo_epi16(tmp1, mask);
+            mask = _mm_cmplt_epi32(tmp2, _mm_set1_epi32(0));
+            __m128i res = _mm_unpacklo_epi32(tmp2, mask);
 #endif
-        return *this;
-    }
+            return res;
+        }
 
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_unaligned(const int8_t* src)
-    {
-        return load_aligned(src);
-    }
-
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_aligned(const uint8_t* src)
-    {
-        __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
+        inline __m128i load_aligned_int64(const uint8_t* src)
+        {
+            __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
-        m_value = _mm_cvtepu8_epi64(tmp);
+            __m128i res = _mm_cvtepu8_epi64(tmp);
 #else
-        __m128i tmp1 = _mm_unpacklo_epi8(tmp, _mm_set1_epi8(0));
-        __m128i tmp2 = _mm_unpacklo_epi16(tmp, _mm_set1_epi16(0));
-        m_value = _mm_unpacklo_epi32(tmp, _mm_set1_epi32(0));
+            __m128i tmp1 = _mm_unpacklo_epi8(tmp, _mm_set1_epi8(0));
+            __m128i tmp2 = _mm_unpacklo_epi16(tmp1, _mm_set1_epi16(0));
+            __m128i res = _mm_unpacklo_epi32(tmp2, _mm_set1_epi32(0));
 #endif
-        return *this;
+            return res;
+        }
+
+        inline __m128i load_aligned_int64(const int16_t* src)
+        {
+            __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
+#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
+            __m128i res = _mm_cvtepi16_epi64(tmp);
+#else
+            __m128i mask = _mm_cmplt_epi16(tmp, _mm_set1_epi16(0));
+            __m128i tmp1 = _mm_unpacklo_epi16(tmp, mask);
+            mask = _mm_cmplt_epi32(tmp1, _mm_set1_epi32(0));
+            __m128i res = _mm_unpacklo_epi32(tmp1, mask);
+#endif
+            return res;
+        }
+
+        inline __m128i load_aligned_int64(const uint16_t* src)
+        {
+            __m128i tmp = _mm_loadl_epi64((const __m128i*)src);
+#if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
+            __m128i res = _mm_cvtepu16_epi64(tmp);
+#else
+            __m128i tmp1 = _mm_unpacklo_epi16(tmp, _mm_set1_epi16(0));
+            __m128i res = _mm_unpacklo_epi32(tmp1, _mm_set1_epi32(0));
+#endif
+            return res;
+        }
+
+        inline void store_aligned_int64(__m128i src, int8_t* dst)
+        {
+            alignas(16) int64_t tmp[2];
+            _mm_store_si128((__m128i*)tmp, src);
+            dst[0] = static_cast<int8_t>(tmp[0]);
+            dst[1] = static_cast<int8_t>(tmp[1]);
+        }
+
+        inline void store_aligned_int64(__m128i src, uint8_t* dst)
+        {
+            alignas(16) int64_t tmp[2];
+            _mm_store_si128((__m128i*)tmp, src);
+            dst[0] = static_cast<uint8_t>(tmp[0]);
+            dst[1] = static_cast<uint8_t>(tmp[1]);
+        }
+
+        inline void store_aligned_int64(__m128i src, int16_t* dst)
+        {
+            alignas(16) int64_t tmp[2];
+            _mm_store_si128((__m128i*)tmp, src);
+            dst[0] = static_cast<int16_t>(tmp[0]);
+            dst[1] = static_cast<int16_t>(tmp[1]);
+        }
+
+        inline void store_aligned_int64(__m128i src, uint16_t* dst)
+        {
+            alignas(16) int64_t tmp[2];
+            _mm_store_si128((__m128i*)tmp, src);
+            dst[0] = static_cast<uint16_t>(tmp[0]);
+            dst[1] = static_cast<uint16_t>(tmp[1]);
+        }
     }
 
-    inline batch<int64_t, 2>& batch<int64_t, 2>::load_unaligned(const uint8_t* src)
-    {
-        return load_aligned(src);
+#define SSE_DEFINE_LOAD_STORE_INT64(TYPE, CVT_TYPE)                            \
+    inline batch<TYPE, 2>& batch<TYPE, 2>::load_aligned(const CVT_TYPE* src)   \
+    {                                                                          \
+        this->m_value = sse_detail::load_aligned_int64(src);                   \
+        return *this;                                                          \
+    }                                                                          \
+    inline batch<TYPE, 2>& batch<TYPE, 2>::load_unaligned(const CVT_TYPE* src) \
+    {                                                                          \
+        return load_aligned(src);                                              \
+    }                                                                          \
+    inline void batch<TYPE, 2>::store_aligned(CVT_TYPE* dst) const             \
+    {                                                                          \
+        sse_detail::store_aligned_int64(this->m_value, dst);                   \
+    }                                                                          \
+    inline void batch<TYPE, 2>::store_unaligned(CVT_TYPE* dst) const           \
+    {                                                                          \
+        store_aligned(dst);                                                    \
     }
 
-    inline void batch<int64_t, 2>::store_aligned(int64_t* dst) const
-    {
-        _mm_store_si128((__m128i*)dst, m_value);
-    }
+    SSE_DEFINE_LOAD_STORE_INT64(int64_t, int8_t)
+    SSE_DEFINE_LOAD_STORE_INT64(int64_t, uint8_t)
+    SSE_DEFINE_LOAD_STORE_INT64(int64_t, int16_t)
+    SSE_DEFINE_LOAD_STORE_INT64(int64_t, uint16_t)
+    XSIMD_DEFINE_LOAD_STORE(int64_t, 2, int32_t, 16)
+    XSIMD_DEFINE_LOAD_STORE(int64_t, 2, uint32_t, 16)
+    XSIMD_DEFINE_LOAD_STORE(int64_t, 2, float, 16)
+    XSIMD_DEFINE_LOAD_STORE(int64_t, 2, double, 16)
 
-    inline void batch<int64_t, 2>::store_unaligned(int64_t* dst) const
-    {
-        _mm_storeu_si128((__m128i*)dst, m_value);
-    }
+    SSE_DEFINE_LOAD_STORE_INT64(uint64_t, int8_t)
+    SSE_DEFINE_LOAD_STORE_INT64(uint64_t, uint8_t)
+    SSE_DEFINE_LOAD_STORE_INT64(uint64_t, int16_t)
+    SSE_DEFINE_LOAD_STORE_INT64(uint64_t, uint16_t)
+    XSIMD_DEFINE_LOAD_STORE(uint64_t, 2, int32_t, 16)
+    XSIMD_DEFINE_LOAD_STORE(uint64_t, 2, uint32_t, 16)
+    XSIMD_DEFINE_LOAD_STORE(uint64_t, 2, float, 16)
+    XSIMD_DEFINE_LOAD_STORE(uint64_t, 2, double, 16)
 
-    inline void batch<int64_t, 2>::store_aligned(int32_t* dst) const
-    {
-        alignas(16) int64_t tmp[2];
-        store_aligned(tmp);
-        dst[0] = static_cast<int32_t>(tmp[0]);
-        dst[1] = static_cast<int32_t>(tmp[1]);
-    }
-
-    inline void batch<int64_t, 2>::store_unaligned(int32_t* dst) const
-    {
-        store_aligned(dst);
-    }
-
-    inline void batch<int64_t, 2>::store_aligned(float* dst) const
-    {
-        alignas(16) int64_t tmp[2];
-        store_aligned(tmp);
-        dst[0] = float(tmp[0]);
-        dst[1] = float(tmp[1]);
-    }
-
-    inline void batch<int64_t, 2>::store_unaligned(float* dst) const
-    {
-        store_aligned(dst);
-    }
-
-    inline void batch<int64_t, 2>::store_aligned(double* dst) const
-    {
-        alignas(16) int64_t tmp[2];
-        store_aligned(tmp);
-        dst[0] = double(tmp[0]);
-        dst[1] = double(tmp[1]);
-    }
-
-    inline void batch<int64_t, 2>::store_unaligned(double* dst) const
-    {
-        store_aligned(dst);
-    }
-
-    inline void batch<int64_t, 2>::store_aligned(int8_t* dst) const
-    {
-        alignas(16) int64_t tmp[2];
-        store_aligned(tmp);
-        dst[0] = static_cast<char>(tmp[0]);
-        dst[1] = static_cast<char>(tmp[1]);
-    }
-
-    inline void batch<int64_t, 2>::store_unaligned(int8_t* dst) const
-    {
-        store_aligned(dst);
-    }
-
-    inline void batch<int64_t, 2>::store_aligned(uint8_t* dst) const
-    {
-        alignas(16) int64_t tmp[2];
-        store_aligned(tmp);
-        dst[0] = static_cast<unsigned char>(tmp[0]);
-        dst[1] = static_cast<unsigned char>(tmp[1]);
-    }
-
-    inline void batch<int64_t, 2>::store_unaligned(uint8_t* dst) const
-    {
-        store_aligned(dst);
-    }
-
-    inline int64_t batch<int64_t, 2>::operator[](std::size_t index) const
-    {
-        alignas(16) int64_t x[2];
-        store_aligned(x);
-        return x[index & 1];
-    }
+#undef SSE_DEFINE_LOAD_STORE_INT64
 
     namespace detail
     {
@@ -436,7 +300,7 @@ namespace xsimd
 #if XSIMD_X86_INSTR_SET >= XSIMD_X86_SSE4_1_VERSION
                 return _mm_cmpeq_epi64(lhs, rhs);
 #else
-                return detail::cmpeq_epi64_sse2(lhs, rhs);
+                return sse_detail::cmpeq_epi64_sse2(lhs, rhs);
 #endif
             }
 
